#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include <util/delay.h>
#include "font.h"
#include "lcd.h"

static unsigned char cursor_row = 0; /* 0-5 */
static unsigned char cursor_col = 0; /* 0-83 */
static unsigned char buffer[6][84];	

/* Function prototypes */
static void lcd_send(unsigned char data, LcdCmdData cd);

/* Performs IO & LCD controller initialization */
void lcd_init(void) {

    // Pull-up on reset pin
    LCD_PORT |= LCD_RST_PIN;

	// Set output bits on lcd port
	LCD_DDR |= LCD_RST_PIN | LCD_CE_PIN | LCD_DC_PIN | LCD_DATA_PIN | LCD_CLK_PIN;

	// Wait after VCC high for reset (max 30ms)
    _delay_ms(15);

    // Toggle display reset pin
    LCD_PORT &= ~LCD_RST_PIN;
    lcd_delay();
    LCD_PORT |= LCD_RST_PIN;

    // Disable LCD controller
    LCD_PORT |= LCD_CE_PIN;

    lcd_send(0x21, LCD_CMD);  // LCD Extended Commands
    lcd_send(0xC8, LCD_CMD);  // Set LCD Vop(Contrast)
    lcd_send(0x06, LCD_CMD);  // Set Temp coefficent
    lcd_send(0x13, LCD_CMD);  // LCD bias mode 1:48
    lcd_send(0x20, LCD_CMD);  // Standard Commands, Horizontal addressing
    lcd_send(0x0C, LCD_CMD);  // LCD in normal mode

    // Clear lcd
    lcd_clear();
}

/**
  *	Schimba adresa pointerului la pixelul 'addr'
  *  x - numarul coloanei (din 84 posibile
  *	 y - numarul linie ( din 6 posibile)
  */
void lcd_gotoXY(uint8_t x, uint8_t y) {

	lcd_send(0x80 | x , LCD_CMD);
	lcd_send(0x40 | y , LCD_CMD);
	
	cursor_row = y;
	cursor_col = x;

}

void lcd_clear(void) {

	int i, j;
	
	lcd_gotoXY(0, 0);  //start with (0,0) position

	for(i = 0; i < 6; i++) {
		lcd_gotoXY(0, i);
		for(j = 0; j < 84; j++) {
			//lcd_send(0x00, LCD_DATA);
			buffer[i][j] = 0x00;
		}
	}
	
    lcd_gotoXY(0, 0);	//bring the XY position back to (0,0)
      
}

void lcd_putchr(unsigned char ch) {

	unsigned char j;
	
	//buffer[cursor_row][cursor_col] = 0x00;
	for(j = 0; j < 5; j++)
		buffer[cursor_row][cursor_col + j] =  pgm_read_byte(&font5x7[ch - 32][j]) << 1;

	//buffer[cursor_row][cursor_col + 6] = 0x00;

	for(j = 0; j < 7; j++)
		lcd_send(buffer[cursor_row][cursor_col + j], LCD_DATA);

} 

void lcd_str(char str[14], uint8_t len) {

	uint8_t i=0;

    for (i = 0; i < len; i ++)
        lcd_putchr(str[i]);

}

/* Sends data to display controller */
static void lcd_send(unsigned char data, LcdCmdData cd) {

    // Enable display controller (active low)
    LCD_PORT &= ~LCD_CE_PIN;

    // Either command or data
    if(cd == LCD_DATA) {
        LCD_PORT |= LCD_DC_PIN;
    } else {
        LCD_PORT &= ~LCD_DC_PIN;
    }

	for (unsigned char i=0;i<8;i++) {
		// Set the DATA pin value
		if((data>>(7-i)) & 0x01) {
			LCD_PORT |= LCD_DATA_PIN;
		} else {
			LCD_PORT &= ~LCD_DATA_PIN;
		}

		// Toggle the clock
		LCD_PORT |= LCD_CLK_PIN;
		LCD_PORT &= ~LCD_CLK_PIN;
	}

	// Disable display controller
    LCD_PORT |= LCD_CE_PIN;

}

void lcd_update(void) {
	
	int i, j;

	for (i = 0; i < 6; i++) {
		lcd_gotoXY(0, i);
		for (j=0; j < 84; j++)
			lcd_send(buffer[i][j], LCD_DATA);
	}
	
	lcd_gotoXY(0, 0);	//bring the XY position back to (0,0)

}

void lcd_setpixel(unsigned char x, unsigned char y) {

	unsigned char value;
	unsigned char row;
	
	row = y / 8;

	value = buffer[row][x];
	value |= (1 << (y % 8));
	buffer[row][x] = value;

	lcd_gotoXY(x, row);
	lcd_send(value, LCD_DATA);

}

#ifdef _EXTRA_FEATURES_
/* Set display contrast. Note: No change is visible at ambient temperature */
void lcd_contrast(unsigned char contrast) {
	lcd_send(0x21, LCD_CMD);				// LCD Extended Commands
	lcd_send(0x80 | contrast, LCD_CMD);		// Set LCD Vop(Contrast)
	lcd_send(0x20, LCD_CMD);				// LCD std cmds, hori addr mode
}
#endif